// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::ffi;

glib::wrapper! {
    #[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
    pub struct PlayVisualization(Boxed<ffi::GstPlayVisualization>);

    match fn {
        copy => |ptr| ffi::gst_play_visualization_copy(ptr),
        free => |ptr| ffi::gst_play_visualization_free(ptr),
        type_ => || ffi::gst_play_visualization_get_type(),
    }
}

unsafe impl Send for PlayVisualization {}
unsafe impl Sync for PlayVisualization {}
