// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::ffi;
use glib::prelude::*;

glib::wrapper! {
    #[doc(alias = "GstPlayVideoRenderer")]
    pub struct PlayVideoRenderer(Interface<ffi::GstPlayVideoRenderer, ffi::GstPlayVideoRendererInterface>);

    match fn {
        type_ => || ffi::gst_play_video_renderer_get_type(),
    }
}

impl PlayVideoRenderer {
    pub const NONE: Option<&'static PlayVideoRenderer> = None;
}

unsafe impl Send for PlayVideoRenderer {}
unsafe impl Sync for PlayVideoRenderer {}

pub trait PlayVideoRendererExt: IsA<PlayVideoRenderer> + 'static {}

impl<O: IsA<PlayVideoRenderer>> PlayVideoRendererExt for O {}
