// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

use crate::{ffi, PlayStreamInfo};
use glib::translate::*;

glib::wrapper! {
    #[doc(alias = "GstPlaySubtitleInfo")]
    pub struct PlaySubtitleInfo(Object<ffi::GstPlaySubtitleInfo, ffi::GstPlaySubtitleInfoClass>) @extends PlayStreamInfo;

    match fn {
        type_ => || ffi::gst_play_subtitle_info_get_type(),
    }
}

impl PlaySubtitleInfo {
    #[doc(alias = "gst_play_subtitle_info_get_language")]
    #[doc(alias = "get_language")]
    pub fn language(&self) -> Option<glib::GString> {
        unsafe {
            from_glib_none(ffi::gst_play_subtitle_info_get_language(
                self.to_glib_none().0,
            ))
        }
    }
}

unsafe impl Send for PlaySubtitleInfo {}
unsafe impl Sync for PlaySubtitleInfo {}
