// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT
#![allow(deprecated)]

use crate::ffi;
use glib::{prelude::*, translate::*};

glib::wrapper! {
    #[doc(alias = "GstPlayStreamInfo")]
    pub struct PlayStreamInfo(Object<ffi::GstPlayStreamInfo, ffi::GstPlayStreamInfoClass>);

    match fn {
        type_ => || ffi::gst_play_stream_info_get_type(),
    }
}

impl PlayStreamInfo {
    pub const NONE: Option<&'static PlayStreamInfo> = None;
}

unsafe impl Send for PlayStreamInfo {}
unsafe impl Sync for PlayStreamInfo {}

pub trait PlayStreamInfoExt: IsA<PlayStreamInfo> + 'static {
    #[doc(alias = "gst_play_stream_info_get_caps")]
    #[doc(alias = "get_caps")]
    fn caps(&self) -> Option<gst::Caps> {
        unsafe {
            from_glib_none(ffi::gst_play_stream_info_get_caps(const_override(
                self.as_ref().to_glib_none().0,
            )))
        }
    }

    #[doc(alias = "gst_play_stream_info_get_codec")]
    #[doc(alias = "get_codec")]
    fn codec(&self) -> Option<glib::GString> {
        unsafe {
            from_glib_none(ffi::gst_play_stream_info_get_codec(const_override(
                self.as_ref().to_glib_none().0,
            )))
        }
    }

    #[cfg_attr(feature = "v1_26", deprecated = "Since 1.26")]
    #[allow(deprecated)]
    #[doc(alias = "gst_play_stream_info_get_index")]
    #[doc(alias = "get_index")]
    fn index(&self) -> i32 {
        unsafe {
            ffi::gst_play_stream_info_get_index(const_override(self.as_ref().to_glib_none().0))
        }
    }

    #[cfg(feature = "v1_26")]
    #[cfg_attr(docsrs, doc(cfg(feature = "v1_26")))]
    #[doc(alias = "gst_play_stream_info_get_stream_id")]
    #[doc(alias = "get_stream_id")]
    fn stream_id(&self) -> glib::GString {
        unsafe {
            from_glib_none(ffi::gst_play_stream_info_get_stream_id(const_override(
                self.as_ref().to_glib_none().0,
            )))
        }
    }

    #[doc(alias = "gst_play_stream_info_get_stream_type")]
    #[doc(alias = "get_stream_type")]
    fn stream_type(&self) -> glib::GString {
        unsafe {
            from_glib_none(ffi::gst_play_stream_info_get_stream_type(const_override(
                self.as_ref().to_glib_none().0,
            )))
        }
    }

    #[doc(alias = "gst_play_stream_info_get_tags")]
    #[doc(alias = "get_tags")]
    fn tags(&self) -> Option<gst::TagList> {
        unsafe {
            from_glib_none(ffi::gst_play_stream_info_get_tags(const_override(
                self.as_ref().to_glib_none().0,
            )))
        }
    }
}

impl<O: IsA<PlayStreamInfo>> PlayStreamInfoExt for O {}
