// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// from gst-gir-files (https://gitlab.freedesktop.org/gstreamer/gir-files-rs.git)
// DO NOT EDIT

mod play;
pub use self::play::Play;

mod play_audio_info;
pub use self::play_audio_info::PlayAudioInfo;

mod play_media_info;
pub use self::play_media_info::PlayMediaInfo;

mod play_signal_adapter;
pub use self::play_signal_adapter::PlaySignalAdapter;

mod play_stream_info;
pub use self::play_stream_info::PlayStreamInfo;

mod play_subtitle_info;
pub use self::play_subtitle_info::PlaySubtitleInfo;

mod play_video_info;
pub use self::play_video_info::PlayVideoInfo;

mod play_video_overlay_video_renderer;
pub use self::play_video_overlay_video_renderer::PlayVideoOverlayVideoRenderer;

mod play_video_renderer;
pub use self::play_video_renderer::PlayVideoRenderer;

mod play_visualization;
pub use self::play_visualization::PlayVisualization;

mod enums;
pub use self::enums::PlayColorBalanceType;
pub use self::enums::PlayError;
pub(crate) use self::enums::PlayMessage;
pub use self::enums::PlaySnapshotFormat;
pub use self::enums::PlayState;

pub(crate) mod traits {
    pub use super::play_stream_info::PlayStreamInfoExt;
    pub use super::play_video_renderer::PlayVideoRendererExt;
}
